@@ -22,53 +22,54 @@
 #define __APPLICATION_H__
 
 #include "me-tv.h"
-#include <libgnomemm.h>
+#include <gtkmm.h>
+#include <dbus/dbus-glib.h>
+#include "status_icon.h"
+
 #include "device_manager.h"
 #include "channel_manager.h"
 #include "scheduled_recording_manager.h"
-#include "main_window.h"
-#include "status_icon.h"
 #include "stream_manager.h"
+#include "configuration_manager.h"
 
-class Application : public Gnome::Main
+extern ChannelManager				channel_manager;
+extern ScheduledRecordingManager	scheduled_recording_manager;
+extern DeviceManager				device_manager;
+extern StreamManager				stream_manager;
+extern ConfigurationManager			configuration_manager;
+
+class Application
 {
 private:
 	static Application*					current;
 	Glib::RefPtr<Gtk::Builder>			builder;
-	MainWindow*							main_window;
 	StatusIcon*							status_icon;
-	Glib::RefPtr<Gnome::Conf::Client>	client;
-	Glib::ustring						preferred_language;
 	Glib::StaticRecMutex				mutex;
 	guint								timeout_source;
 	Glib::ustring						application_dir;
 	Data::Schema						schema;
 	Glib::ustring						database_filename;
 	gboolean							database_initialised;
-
-	void set_string_configuration_default(const Glib::ustring& key, const Glib::ustring& value);
-	void set_int_configuration_default(const Glib::ustring& key, gint value);
-	void set_boolean_configuration_default(const Glib::ustring& key, gboolean value);	
-	Glib::ustring get_configuration_path(const Glib::ustring& key);
-
+	DBusGConnection*					dbus_connection;
+	
 	void make_directory_with_parents(const Glib::ustring& path);
 		
 	void on_display_channel_changed(const Channel& channel);
 	static gboolean on_timeout(gpointer data);
 	gboolean on_timeout();
-	void on_error(const Glib::ustring& message);
+	void action_after(guint action);
 
+	void on_record_current();
+	void on_quit();
+	
 public:
-	Application(int argc, char *argv[], Glib::OptionContext& option_context);
+	Application();
 	~Application();
-		
+
 	void run();
+	void quit();
 	static Application& get_current();
 	
-	ChannelManager				channel_manager;
-	ScheduledRecordingManager	scheduled_recording_manager;
-	DeviceManager				device_manager;
-	StreamManager				stream_manager;
 	Data::Connection			connection;
 
 	Glib::StaticRecMutex&	get_mutex();
@@ -76,36 +77,17 @@ public:
 	Data::Schema			get_schema() const { return schema; }
 
 	const Glib::ustring& get_database_filename();
-	void update();
-		
-	Glib::ustring	get_string_configuration_value(const Glib::ustring& key);
-	gint			get_int_configuration_value(const Glib::ustring& key);
-	gboolean		get_boolean_configuration_value(const Glib::ustring& key);
-
-	void set_string_configuration_value(const Glib::ustring& key, const Glib::ustring& value);
-	void set_int_configuration_value(const Glib::ustring& key, gint value);
-	void set_boolean_configuration_value(const Glib::ustring& key, gboolean value);
 	
 	Glib::RefPtr<Gtk::Builder> get_builder() { return builder; }
 	
-	void set_display_channel(const Channel& channel);
-	void set_display_channel_by_id(guint channel_id);
-	void set_display_channel_number(guint display_channel_number);
-	void previous_channel();
-	void next_channel();
-	
+	void check_auto_record();
 	void check_scheduled_recordings();
-	void on_record();
-	void start_recording(Channel& channel, const Glib::ustring& description = "", gboolean scheduled = false);
+	void start_recording(Channel& channel);
+	void start_recording(Channel& channel, const ScheduledRecording& scheduled_recording);
 	void stop_recording(Channel& channel);
-	Glib::ustring make_recording_filename(Channel& channel, const Glib::ustring& description = "");
 		
-	const Glib::ustring& get_preferred_language() const { return preferred_language; }
 	const Glib::ustring& get_application_dir() const { return application_dir; }
-	
-	MainWindow& get_main_window();
-
-	void select_channel_to_play();
+	DBusGConnection* get_dbus_connection() const { return dbus_connection; }
 };
 
 Application& get_application();
