@@ -37,13 +37,12 @@ Dvb::SI::Event::Event()
 	event_id = 0;
 	start_time = 0;
 	duration = 0;
-	running_status = 0;
-	free_CA_mode = 0;
+	version_number = 0;
 }
 
 SectionParser::SectionParser()
 {
-	text_encoding = get_application().get_string_configuration_value("text_encoding");
+	text_encoding = configuration_manager.get_string_value("text_encoding");
 }
 
 // this method takes a bitmask as supplied by the NIT-section (satellite delivery system descriptor) for the FEC_INNER and converts them to a linux-style FEC value according to EN 300 468, v 1.9.1
@@ -86,8 +85,16 @@ void SectionParser::parse_sds (Demuxer& demuxer, ServiceDescriptionSection& sect
 	while (offset < section_length - 4)
 	{
 		Service service;
+
 		service.id = buffer.get_bits(offset, 0, 16);
-		offset += 3;
+		offset += 2;
+		service.eit_schedule_flag = buffer.get_bits(offset, 6, 1) == 1;
+		if (service.eit_schedule_flag)
+		{
+			section.epg_events_available = true;
+		}
+		offset++;
+
 		guint descriptors_loop_length = buffer.get_bits(offset, 4, 12);
 		offset += 2;
 		guint descriptors_end_offset = offset + descriptors_loop_length;
@@ -455,17 +462,19 @@ void SectionParser::parse_psip_eis(Demuxer& demuxer, EventInformationSection& se
 	guint offset = 3;
 
 	section.service_id = buffer.get_bits(offset, 0, 16);
+	section.version_number = buffer.get_bits(42, 5);
 	
 	offset += 6;
 	guint num_events_in_section = buffer[offset++];
 	
-	for (guint i = 0; i < num_events_in_section; i++)
+	for (guint i = 0; i < num_events_in_section && (offset < (section_length - CRC_BYTE_SIZE)); i++)
 	{
 		Event event;
 
-		event.event_id		= buffer.get_bits(offset, 2, 14); offset += 2;
-		event.start_time	= buffer.get_bits(offset, 0, 32); offset += 4;
-		event.duration		= buffer.get_bits(offset, 4, 20); offset += 3;
+		event.version_number	= section.version_number;
+		event.event_id			= buffer.get_bits(offset, 2, 14); offset += 2;
+		event.start_time		= buffer.get_bits(offset, 0, 32); offset += 4;
+		event.duration			= buffer.get_bits(offset, 4, 20); offset += 3;
 
 		event.start_time += GPS_EPOCH;
 		event.start_time += timezone;
@@ -481,7 +490,7 @@ void SectionParser::parse_psip_eis(Demuxer& demuxer, EventInformationSection& se
 		
 		guint descriptors_length = buffer.get_bits(offset, 4, 12);
 		offset += 2 + descriptors_length;
-		
+	
 		section.events.push_back(event);
 	}
 }
@@ -514,12 +523,11 @@ void SectionParser::parse_eis(Demuxer& demuxer, EventInformationSection& section
 		gulong	start_time_UTC;  // 24
 		gulong	duration;
 
+		event.version_number	= section.version_number;
 		event.event_id			= buffer.get_bits(offset, 0, 16);
 		start_time_MJD			= buffer.get_bits(offset, 16, 16);
 		start_time_UTC			= buffer.get_bits(offset, 32, 24);
 		duration				= buffer.get_bits(offset, 56, 24);
-		event.running_status	= buffer.get_bits(offset, 80, 3);
-		event.free_CA_mode		= buffer.get_bits(offset, 83, 1);
 		
 		unsigned int descriptors_loop_length  = buffer.get_bits(offset, 84, 12);
 		offset += 12;
@@ -530,36 +538,23 @@ void SectionParser::parse_eis(Demuxer& demuxer, EventInformationSection& section
 		guint event_dur_sec =  ((duration >>  4)&0xf)*10+((duration      )&0xf);
 
 		event.duration = (event_dur_hour*60 + event_dur_min) * 60 + event_dur_sec;
-		
-		guint event_start_day = 0;
-		guint event_start_month = 0;
-		guint event_start_year = 0;
-		
+				
 		if (start_time_MJD > 0 && event.event_id != 0)
 		{
 			long year =  (long) ((start_time_MJD  - 15078.2) / 365.25);
 			long month =  (long) ((start_time_MJD - 14956.1 - (long)(year * 365.25) ) / 30.6001);
-			event_start_day =  (long) (start_time_MJD - 14956 - (long)(year * 365.25) - (long)(month * 30.6001));
-
-			long k = (month == 14 || month == 15) ? 1 : 0;
-
-			event_start_year = year + k + 1900;
-			event_start_month = month - 1 - k * 12;
-
-			guint event_start_hour = ((start_time_UTC >> 20)&0xf)*10+((start_time_UTC >> 16)&0xf);
-			guint event_start_min =  ((start_time_UTC >> 12)&0xf)*10+((start_time_UTC >>  8)&0xf);
-			guint event_start_sec =  ((start_time_UTC >>  4)&0xf)*10+((start_time_UTC      )&0xf);
+			long day =  (long) (start_time_MJD - 14956 - (long)(year * 365.25) - (long)(month * 30.6001));
 
 			struct tm t;
 
 			memset(&t, 0, sizeof(struct tm));
 
-			t.tm_sec	= event_start_sec;
-			t.tm_min	= event_start_min;
-			t.tm_hour	= event_start_hour;
-			t.tm_mday	= event_start_day;
-			t.tm_mon	= event_start_month - 1;
-			t.tm_year	= event_start_year - 1900;
+			t.tm_sec	= ((start_time_UTC >>  4)&0xf)*10+((start_time_UTC      )&0xf);
+			t.tm_min	= ((start_time_UTC >> 12)&0xf)*10+((start_time_UTC >>  8)&0xf);
+			t.tm_hour	= ((start_time_UTC >> 20)&0xf)*10+((start_time_UTC >> 16)&0xf);
+			t.tm_mday	= day;
+			t.tm_mon	= month - 2;
+			t.tm_year	= year;
 			
 			event.start_time = mktime(&t);
 
@@ -608,13 +603,7 @@ gsize SectionParser::decode_event_descriptor (const guchar* event_buffer, Event&
 	{		
 	case EXTENDED_EVENT:
 		{
-			Glib::ustring language;
-			Glib::ustring title;
-			Glib::ustring description;
-			Glib::ustring temp_title;
-			Glib::ustring temp_description;
-
-			language = get_lang_desc (event_buffer + 1);
+			Glib::ustring language = get_lang_desc (event_buffer + 1);
 			
 			if (!event.texts.contains(language))
 			{
@@ -625,46 +614,20 @@ gsize SectionParser::decode_event_descriptor (const guchar* event_buffer, Event&
 			EventText& event_text = event.texts[language];
 			
 			unsigned int offset = 6;
-			guint length_of_items = event_buffer[offset];
-			offset++;
+			guint length_of_items = event_buffer[offset++];
 			while (length_of_items > offset - 7)
 			{
-				offset += get_text(temp_description, &event_buffer[offset]);			
-				if (temp_description != "-")
-				{
-					description += temp_description;
-				}
-
-				offset += get_text(temp_title, &event_buffer[offset]);
-				if (temp_title != "-")
-				{
-					title += temp_title;
-				}
+				offset += get_text(event_text.title, &event_buffer[offset]);
+				offset += get_text(event_text.description, &event_buffer[offset]);
 			}
-			offset += get_text(temp_description, &event_buffer[offset]);
-			if (temp_description != "-")
-			{
-				description	+= temp_description;
-			}
-			
-			if (!event_text.is_extended)
-			{
-				event_text.description.clear();
-			}
-			
-			event_text.is_extended	= true;
-			event_text.title		+= title;
-			event_text.description	+= description;
+			offset += get_text(event_text.description, &event_buffer[offset]);
 		}
 		break;
 
 	case SHORT_EVENT:
 		{
-			Glib::ustring	language;
-			Glib::ustring	title;
-			Glib::ustring	description;
+			Glib::ustring language = get_lang_desc (event_buffer);
 			
-			language = get_lang_desc (event_buffer);
 			if (!event.texts.contains(language))
 			{
 				EventText event_text_temp;
@@ -673,18 +636,10 @@ gsize SectionParser::decode_event_descriptor (const guchar* event_buffer, Event&
 			}
 			EventText& event_text = event.texts[language];
 
-			event_text.is_extended = false;
 			event_text.language = language;
 			unsigned int offset = 5;
-			offset += get_text(title, &event_buffer[offset]);
-			offset += get_text(description, &event_buffer[offset]);
-			
-			event_text.title = title;
-
-			if (!event_text.is_extended)
-			{
-				event_text.description += description;
-			}
+			offset += get_text(event_text.title, &event_buffer[offset]);
+			offset += get_text(event_text.subtitle, &event_buffer[offset]);
 		}
 		break;
 	default:
@@ -782,11 +737,9 @@ gsize SectionParser::get_text(Glib::ustring& s, const guchar* text_buffer)
 			}
 		}
 		
-		s = "";
-
 		if (text_encoding == "iso6937")
 		{
-			s = convert_iso6937(text_buffer + index, length);
+			s += convert_iso6937(text_buffer + index, length);
 		}
 		else
 		{
@@ -865,7 +818,7 @@ gsize SectionParser::get_text(Glib::ustring& s, const guchar* text_buffer)
 					throw Exception(message);
 				}
 				
-				s = result;
+				s += result;
 				g_free(result);
 			}
 		}
