@@ -32,6 +32,8 @@
 #include "me-tv-i18n.h"
 
 #define DVB_SECTION_BUFFER_SIZE	16*1024
+#define TS_PACKET_SIZE			188
+#define PACKET_BUFFER_SIZE		50
 
 #define PAT_PID		0x00
 #define NIT_PID		0x10
@@ -57,9 +59,9 @@ namespace Dvb
 		class EventText
 		{
 		public:
-			gboolean is_extended;
 			Glib::ustring language;
 			Glib::ustring title;
+			Glib::ustring subtitle;
 			Glib::ustring description;
 		};
 		
@@ -75,10 +77,9 @@ namespace Dvb
 			Event();
 
 			guint	event_id;
+			guint	version_number;
 			guint	start_time;
 			gulong	duration;
-			guint	running_status;
-			guint	free_CA_mode;
 			
 			EventTextMap texts;
 		};
@@ -108,6 +109,7 @@ namespace Dvb
 		public:
 			guint id;
 			guint type;
+			gboolean eit_schedule_flag;
 			Glib::ustring provider_name;
 			Glib::ustring name;
 		};
@@ -116,6 +118,7 @@ namespace Dvb
 		{
 		public:
 			guint transport_stream_id;
+			gboolean epg_events_available;
 			std::vector<Service> services;
 		};
 
