@@ -28,6 +28,7 @@ using namespace Dvb;
 
 Transponder::Transponder() : polarisation(0), satellite_number(0), hi_band(0)
 {
+	frontend_type = FE_OFDM;
 	memset(&frontend_parameters, 0, sizeof(frontend_parameters));
 }
 
@@ -54,10 +55,24 @@ void TransponderList::add(const Transponder& transponder)
 
 bool Transponder::operator==(const Transponder& transponder) const
 {
-	return transponder.frontend_parameters.frequency == frontend_parameters.frequency;
+	return transponder == frontend_parameters;
 }
 
 bool Transponder::operator!=(const Transponder& transponder) const
 {
-	return transponder.frontend_parameters.frequency != frontend_parameters.frequency;
+	return transponder != frontend_parameters;
+}
+
+bool Transponder::operator==(struct dvb_frontend_parameters p) const
+{
+	return frontend_parameters.frequency == p.frequency && (
+		(frontend_type == FE_OFDM && frontend_parameters.u.ofdm.bandwidth == p.u.ofdm.bandwidth) ||
+		(frontend_type == FE_ATSC && frontend_parameters.u.vsb.modulation == p.u.vsb.modulation) ||
+		(frontend_type == FE_QAM && frontend_parameters.u.qam.modulation == p.u.qam.modulation)
+	);
+}
+
+bool Transponder::operator!=(struct dvb_frontend_parameters p) const
+{
+	return !(*this == p);
 }
