@@ -21,14 +21,14 @@
 #include "engine.h"
 #include "exception.h"
 #include "application.h"
-#include <libgnome/libgnome.h>
+#include <gtkmm.h>
 #include <gdk/gdkx.h>
 #include <sys/wait.h>
 
 #define KILL_SLEEP_TIME		100000
 #define KILL_SLEEP_TIMEOUT  2000000
 
-Engine::Engine(const Glib::ustring& engine_type)
+Engine::Engine()
 {
 	pid = -1;
 	mute_state = false;
@@ -36,7 +36,6 @@ Engine::Engine(const Glib::ustring& engine_type)
 	audio_channel_state = AUDIO_CHANNEL_STATE_BOTH;
 	audio_stream = 0;
 	subtitle_stream = -1;
-	type = engine_type;
 
 	Gtk::DrawingArea* drawing_area_video = NULL;
 	get_application().get_builder()->get_widget("drawing_area_video", drawing_area_video);
@@ -61,13 +60,16 @@ void Engine::play(const Glib::ustring& mrl)
 	g_debug("Engine::play(\"%s\")", mrl.c_str());
 
 	StringList argv;
-	argv.push_back("me-tv-" + type);
+	argv.push_back("me-tv-player");
 	argv.push_back(Glib::ustring::compose("fifo://%1", mrl));
 	argv.push_back(Glib::ustring::compose("%1", window));
-	argv.push_back(application.get_string_configuration_value("xine.video_driver"));
-	argv.push_back(application.get_string_configuration_value("xine.audio_driver"));
-	argv.push_back(application.get_string_configuration_value("deinterlace_type"));
+	argv.push_back(configuration_manager.get_string_value("video_driver"));
+	argv.push_back(configuration_manager.get_string_value("audio_driver"));
+	argv.push_back(configuration_manager.get_string_value("deinterlace_type"));
 	argv.push_back(mute_state ? "true" : "false");
+	argv.push_back(Glib::ustring::compose("%1", audio_stream));
+	argv.push_back(Glib::ustring::compose("%1", subtitle_stream));
+	argv.push_back(Glib::ustring::compose("%1", (int)(volume * 100)));
 
 	g_debug("=================================================");
 	for (StringList::iterator i = argv.begin(); i != argv.end(); i++)
@@ -176,7 +178,7 @@ void Engine::sendKeyEvent(int keycode, int modifiers)
 	event.x_root      = 1;
 	event.y_root      = 1;
 	event.same_screen = True;
-	event.keycode     = XKeysymToKeycode(display, keycode);
+	event.keycode     = keycode;
 	event.state       = modifiers;
 
 	event.type = KeyPress;
@@ -194,7 +196,7 @@ void Engine::pause(gboolean state)
 		g_debug(state ? "Sending pause" : "Sending unpause");
 		sendKeyEvent(
 			XK_space,
-			state ? XK_Shift_L : 0);
+			state ? XK_Control_L : 0);
 	}
 }
 
@@ -209,7 +211,7 @@ void Engine::set_mute_state(gboolean state)
 			g_debug(state ? "Sending mute" : "Sending unmute");
 			sendKeyEvent(
 				XK_m,
-				state ? 0 : XK_Shift_L);
+				state ? 0 : XK_Control_L);
 		}
 	}
 }
@@ -225,15 +227,15 @@ void Engine::set_audio_channel_state(AudioChannelState state)
 			switch(state)
 			{					
 			case AUDIO_CHANNEL_STATE_LEFT:
-				modifiers &= XK_Shift_L;
+				modifiers = XK_Control_L;
 				break;
 				
 			case AUDIO_CHANNEL_STATE_RIGHT:
-				modifiers &= XK_Shift_R;
+				modifiers = XK_Control_R;
 				break;
 
 			default:
-				modifiers &= XK_Shift_L & XK_Shift_R;
+				modifiers = XK_Control_L & XK_Control_R;
 				break;
 			}
 			sendKeyEvent(XK_a, modifiers);
@@ -241,14 +243,26 @@ void Engine::set_audio_channel_state(AudioChannelState state)
 	}
 }
 
-void Engine::set_audio_stream(guint stream)
+void Engine::set_volume(float value)
+{
+	g_debug("Setting volume: %f", value);
+	volume = value;
+	
+	if (pid != -1)
+	{
+		// At value = 1.0 the key will be a colon (XK_colon)
+		sendKeyEvent(XK_0 + (int)(value * 10), XK_Control_L & XK_Control_R);
+	}
+}
+
+void Engine::set_audio_stream(gint stream)
 {
 	if (audio_stream != stream)
 	{
 		audio_stream = stream;
 		if (pid != -1)
 		{
-			sendKeyEvent(XK_1 + stream, XK_Shift_L);
+			sendKeyEvent(XK_0 + stream, XK_Control_L);
 		}
 	}
 }
@@ -260,8 +274,7 @@ void Engine::set_subtitle_stream(gint stream)
 		subtitle_stream = stream;
 		if (pid != -1)
 		{
-			sendKeyEvent(XK_1 + stream, XK_Shift_R);
+			sendKeyEvent(XK_0 + stream, XK_Control_R);
 		}
 	}
 }
-
