@@ -22,7 +22,7 @@
 #define __ENGINE_H__
 
 #include "me-tv.h"
-#include <libgnomeuimm.h>
+#include <gtkmm.h>
 #include <X11/X.h>
 
 class Engine
@@ -40,10 +40,10 @@ private:
 	gboolean			mute_state;
 	gboolean			deinterlacer_state;
 	AudioChannelState	audio_channel_state;
-	guint				audio_stream;
+	gint				audio_stream;
 	gint				subtitle_stream;
+	float				volume;
 	Glib::ustring		mrl;
-	Glib::ustring		type;
 	Window				window;
 
 	void sendKeyEvent(int keycode, int modifiers);
@@ -53,12 +53,13 @@ public:
 	void stop();
 	void pause(gboolean state);
 	void set_mute_state(gboolean state);
-	void set_audio_stream(guint stream);
-	void set_audio_channel_state(AudioChannelState state);
+	void set_volume(float value);
 	void set_subtitle_stream(gint stream);
+	void set_audio_stream(gint stream);
+	void set_audio_channel_state(AudioChannelState state);
 	gboolean is_running();
 
-	Engine(const Glib::ustring& type);
+	Engine();
 	~Engine();
 };
 
