@@ -21,8 +21,8 @@
 #ifndef __EPG_EVENTS_H__
 #define __EPG_EVENTS_H__
 
-#include "epg_event.h"
 #include "data.h"
+#include "epg_event.h"
 
 typedef std::list<EpgEvent> EpgEventList;
 
@@ -31,20 +31,21 @@ class EpgEvents
 private:
 	EpgEventList			list;
 	Glib::StaticRecMutex	mutex;
+	gboolean				dirty;
 	
-	gboolean exists(const EpgEvent& epg_event);
 	void set_saved(guint epg_event_id);
 public:
 	EpgEvents();
 	~EpgEvents();
 		
-	gboolean			add_epg_event(const EpgEvent& epg_event);
-	void				add_epg_events(const EpgEventList& epg_event_list);
-	gboolean			get_current(EpgEvent& epg_event);
-	const EpgEventList	get_list();
-	void				prune();
-	void				load(Data::Connection& connection, guint channel_id);
-	void				save(Data::Connection& connection, guint channel_id);
+	gboolean		add_epg_event(const EpgEvent& epg_event);
+	gboolean		get_current(EpgEvent& epg_event);
+	EpgEventList	get_list(guint start_time, guint end_time);
+	void			prune();
+	void			load(Data::Connection& connection, guint channel_id);
+	void			save(Data::Connection& connection, guint channel_id);
+	EpgEventList	search(const Glib::ustring& text, gboolean search_description);
+	gboolean		is_dirty() const { return dirty; }
 };
 
 #endif
