@@ -21,6 +21,7 @@
 #include "gtk_epg_widget.h"
 #include "application.h"
 #include "scheduled_recording_dialog.h"
+#include "epg_event_dialog.h"
 
 #define MINUTES_PER_COLUMN	1
 #define COLUMNS_PER_HOUR	(60 * MINUTES_PER_COLUMN)
@@ -29,12 +30,11 @@ GtkEpgWidget::GtkEpgWidget(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Buil
 	Gtk::ScrolledWindow(cobject), builder(builder)
 {
 	offset = 0;
-	epg_page = 0;
 
 	builder->get_widget("table_epg", table_epg);
 	builder->get_widget("scrolled_window_epg", scrolled_window_epg);
 	
-	epg_span_hours = get_application().get_int_configuration_value("epg_span_hours");
+	epg_span_hours = configuration_manager.get_int_value("epg_span_hours");
 	Gtk::Button* button = NULL;
 	
 	builder->get_widget("button_epg_now", button);
@@ -47,8 +47,8 @@ GtkEpgWidget::GtkEpgWidget(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Buil
 	button->signal_clicked().connect(sigc::mem_fun(*this, &GtkEpgWidget::next));
 
 	builder->get_widget("label_epg_page", label_epg_page);
-	builder->get_widget_derived("combo_box_epg_page", combo_box_epg_page);
-	combo_box_epg_page->signal_changed().connect(sigc::mem_fun(*this, &GtkEpgWidget::on_combo_box_epg_page_changed));
+	builder->get_widget("spin_button_epg_page", spin_button_epg_page);
+	spin_button_epg_page->signal_changed().connect(sigc::mem_fun(*this, &GtkEpgWidget::on_spin_button_epg_page_changed));
 }
 
 void GtkEpgWidget::set_offset(gint value)
@@ -72,22 +72,9 @@ void GtkEpgWidget::next()
 	set_offset(offset + (epg_span_hours*60*60));
 }
 
-void GtkEpgWidget::on_combo_box_epg_page_changed()
+void GtkEpgWidget::on_spin_button_epg_page_changed()
 {	
-	TRY
-	if (combo_box_epg_page->get_size() > 0)
-	{
-		try
-		{
-			epg_page = combo_box_epg_page->get_active_value();
-		}
-		catch(...)
-		{
-			g_debug("Ignoring invalid active integer value");
-		}
-		update_table();
-	}
-	CATCH
+	update_table();
 }
 
 void GtkEpgWidget::update()
@@ -115,31 +102,28 @@ void GtkEpgWidget::clear()
 
 void GtkEpgWidget::update_pages()
 {
-	Application& application = get_application();
-	guint epg_page_count = combo_box_epg_page->get_size();
-	guint epg_page_size = application.get_int_configuration_value("epg_page_size");
-
+	guint epg_page_size = configuration_manager.get_int_value("epg_page_size");
 	if (epg_page_size == 0)
 	{
 		return;
 	}
+
+	double min = 0, max = 0;
+	spin_button_epg_page->get_range(min, max);
 	
-	const ChannelArray& channels = application.channel_manager.get_channels();
+	guint epg_page_count = max;
+	
+	const ChannelArray& channels = channel_manager.get_channels();
 	guint channel_count = channels.size();
 	guint new_epg_page_count = channel_count == 0 ? 1 : ((channel_count-1) / epg_page_size) + 1;
 	
-	if (epg_page == 0 && new_epg_page_count >= 1)
-	{
-		epg_page = 1;
-	}
-	
 	if (new_epg_page_count != epg_page_count)
 	{		
-		combo_box_epg_page->set_size(new_epg_page_count);
+		spin_button_epg_page->set_range(1, new_epg_page_count);
 	}
 	
 	label_epg_page->property_visible() = new_epg_page_count > 1;
-	combo_box_epg_page->property_visible() = new_epg_page_count > 1;
+	spin_button_epg_page->property_visible() = new_epg_page_count > 1;
 }
 
 void GtkEpgWidget::update_table()
@@ -156,7 +140,7 @@ void GtkEpgWidget::update_table()
 		
 		clear();
 		
-		epg_span_hours = get_application().get_int_configuration_value("epg_span_hours");
+		epg_span_hours = configuration_manager.get_int_value("epg_span_hours");
 
 		Gtk::Widget* widget = NULL;
 
@@ -166,12 +150,6 @@ void GtkEpgWidget::update_table()
 		builder->get_widget("button_epg_now", widget);
 		widget->set_sensitive(offset > 0);
 		
-		ChannelManager& channel_manager = get_application().channel_manager;
-		guint display_channel_index = -1;
-		if (channel_manager.has_display_channel())
-		{
-			display_channel_index = channel_manager.get_display_channel_index();
-		}
 		ChannelArray& channels = channel_manager.get_channels();
 
 		table_epg->resize(epg_span_hours * COLUMNS_PER_HOUR + 1, channels.size() + 1);
@@ -180,24 +158,35 @@ void GtkEpgWidget::update_table()
 		start_time = (start_time / COLUMNS_PER_HOUR) * COLUMNS_PER_HOUR;
 		
 		guint row = 0;
-		gboolean show_epg_header = get_application().get_boolean_configuration_value("show_epg_header");
+		gboolean show_epg_header = configuration_manager.get_boolean_value("show_epg_header");
 		if (show_epg_header)
 		{
 			for (guint hour = 0; hour < epg_span_hours; hour++)
 			{
 				guint hour_time = start_time + (hour * 60 * 60);
 				Glib::ustring hour_time_text = get_local_time_text(hour_time, "%c");
-				Gtk::Button& button = attach_button(hour_time_text, false, hour * COLUMNS_PER_HOUR + 1, (hour+1) * COLUMNS_PER_HOUR + 1, 0, 1, Gtk::FILL | Gtk::EXPAND);
-				button.set_sensitive(false);
+
+				Gtk::Frame* frame = Gtk::manage(new Gtk::Frame());
+				Gtk::Label* label = Gtk::manage(new Gtk::Label(hour_time_text));
+				label->set_alignment(0,0.5);
+				label->show();
+				frame->add(*label);
+				frame->set_shadow_type(Gtk::SHADOW_OUT);
+				
+				attach_widget(*frame,
+				    hour * COLUMNS_PER_HOUR + 1,
+				    (hour+1) * COLUMNS_PER_HOUR + 1,
+				    0, 1, Gtk::FILL | Gtk::EXPAND);
 			}
 			row++;
 		}
 		start_time += timezone;
 
-		guint epg_page_size = get_application().get_int_configuration_value("epg_page_size");
-		gboolean show_channel_number = get_application().get_boolean_configuration_value("show_channel_number");
-		gboolean show_epg_time = get_application().get_boolean_configuration_value("show_epg_time");
-		gboolean show_epg_tooltips = get_application().get_boolean_configuration_value("show_epg_tooltips");
+		gint epg_page = spin_button_epg_page->is_visible() ? spin_button_epg_page->get_value_as_int() : 1;
+		guint epg_page_size = configuration_manager.get_int_value("epg_page_size");
+		gboolean show_channel_number = configuration_manager.get_boolean_value("show_channel_number");
+		gboolean show_epg_time = configuration_manager.get_boolean_value("show_epg_time");
+		gboolean show_epg_tooltips = configuration_manager.get_boolean_value("show_epg_tooltips");
 		guint channel_start = (epg_page-1) * epg_page_size;
 		guint channel_end = channel_start + epg_page_size;
 
@@ -205,12 +194,13 @@ void GtkEpgWidget::update_table()
 		{
 			channel_end = channels.size();
 		}
-		
+
+		Gtk::RadioButtonGroup group;
 		for (guint channel_index = channel_start; channel_index < channel_end; channel_index++)
 		{
 			Channel& channel = channels[channel_index];
-			gboolean selected = channel_index == display_channel_index;
-			create_channel_row(channel, row++, selected, start_time, channel_index + 1,
+			gboolean selected = stream_manager.has_display_stream() && (stream_manager.get_display_channel() == channel);
+			create_channel_row(group, channel, row++, selected, start_time, channel_index + 1,
 				show_channel_number, show_epg_time, show_epg_tooltips);
 		}
 		get_window()->thaw_updates();
@@ -219,21 +209,19 @@ void GtkEpgWidget::update_table()
 	vadjustment->set_value(vvalue);
 }
 
-void GtkEpgWidget::create_channel_row(const Channel& const_channel,
+void GtkEpgWidget::create_channel_row(Gtk::RadioButtonGroup& group, Channel& channel,
 	guint table_row, gboolean selected, guint start_time, guint channel_number,
 	gboolean show_channel_number, gboolean show_epg_time, gboolean show_epg_tooltips)
-{	
-	Channel channel = const_channel;
-
+{
 	Glib::ustring channel_text = Glib::ustring::compose("<b>%1</b>", encode_xml(channel.name));
 	if (show_channel_number)
 	{
 		channel_text = Glib::ustring::compose("<i>%1.</i> ", channel_number) + channel_text;
 	}
 	
-	gboolean record_channel = get_application().stream_manager.is_recording(const_channel);
+	gboolean record_channel = stream_manager.is_recording(channel);
 	
-	Gtk::ToggleButton& channel_button = attach_toggle_button(channel_text, record_channel, 0, 1, table_row, table_row + 1);
+	Gtk::RadioButton& channel_button = attach_radio_button(group, channel_text, record_channel, 0, 1, table_row, table_row + 1);
 
 	if (selected)
 	{
@@ -245,18 +233,11 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 	}
 	
 	channel_button.set_active(selected);
-	channel_button.signal_clicked().connect(
+	channel_button.signal_button_press_event().connect(
 		sigc::bind<guint>
-		(
-			sigc::mem_fun(*this, &GtkEpgWidget::on_button_channel_clicked),
-			channel.channel_id
-		)
-	);
-	channel_button.signal_button_press_event().connect(
-		sigc::bind<Channel>
 		(
 			sigc::mem_fun(*this, &GtkEpgWidget::on_button_channel_press_event),
-			channel
+			channel.channel_id
 		),
 	    false
 	);
@@ -268,92 +249,99 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 
 	if (!disable_epg)
 	{
-		EpgEventList events = channel.epg_events.get_list();
-		for (EpgEventList::const_iterator i = events.begin(); i != events.end(); i++)
+		EpgEventList events = channel.epg_events.get_list(start_time, end_time);
+		for (EpgEventList::iterator i = events.begin(); i != events.end(); i++)
 		{
-			const EpgEvent& epg_event = *i;
-					
-			if (
-				(epg_event.start_time >= start_time && epg_event.start_time <= end_time) ||
-				(epg_event.get_end_time() >= start_time && epg_event.get_end_time() <= end_time) ||
-				(epg_event.start_time <= start_time && epg_event.get_end_time() >= end_time)
-			)
+			EpgEvent& epg_event = *i;
+
+			guint event_end_time = epg_event.start_time + epg_event.duration;		
+			guint start_column = 0;
+			if (epg_event.start_time < start_time)
 			{
-				guint event_end_time = epg_event.start_time + epg_event.duration;		
-				guint start_column = 0;
-				if (epg_event.start_time < start_time)
-				{
-					start_column = 0;
-				}
-				else
-				{
-					start_column = (guint)round((epg_event.start_time - start_time) / COLUMNS_PER_HOUR);
-				}
-			
-				guint end_column = (guint)round((event_end_time - start_time) / COLUMNS_PER_HOUR);
-				if (end_column > number_columns-1)
-				{
-					end_column = number_columns-1;
-				}
-			
-				guint column_count = end_column - start_column;
-				if (start_column >= total_number_columns && column_count > 0)
-				{
-					// If there's a gap, plug it
-					if (start_column > total_number_columns)
-					{
-						guint empty_columns = start_column - total_number_columns;
-						Gtk::Button& button = attach_button(
-							empty_columns < 10 ? _("-") : _("Unknown program"), false,
-							total_number_columns + 1, start_column + 1, table_row, table_row + 1);
-						button.set_sensitive(false);
-						total_number_columns += empty_columns;
-					}
-				
-					if (column_count > 0)
-					{
-						guint converted_start_time = convert_to_utc_time(epg_event.start_time);
-
-						Glib::ustring text;
-						if (show_epg_time)
-						{
-							text = get_local_time_text(converted_start_time, "<b>%H:%M");
-							text += get_local_time_text(converted_start_time + epg_event.duration, " - %H:%M</b>\n");
-						}
-						text += encode_xml(epg_event.get_title());
-
-						gboolean record = get_application().scheduled_recording_manager.is_recording(epg_event);
-						
-						Gtk::Button& button = attach_button(text, record, start_column + 1, end_column + 1, table_row, table_row + 1);
-						button.signal_clicked().connect(
-							sigc::bind<EpgEvent>
-							(
-								sigc::mem_fun(*this, &GtkEpgWidget::on_button_program_clicked),
-								epg_event
-							)
-						);
-						button.signal_button_press_event().connect(
-							sigc::bind<EpgEvent>
-							(
-								sigc::mem_fun(*this, &GtkEpgWidget::on_button_program_press_event),
-								epg_event
-							),
-						    false
-						);
-
-
-						if (show_epg_tooltips)
-						{
-							Glib::ustring tooltip_text = get_local_time_text(converted_start_time, "%A, %B %d\n%H:%M");
-							tooltip_text += get_local_time_text(converted_start_time + epg_event.duration, " - %H:%M");
-							button.set_tooltip_text(tooltip_text);
-						}
-					}
-
-					total_number_columns += column_count;
-				}
-				last_event_end_time = event_end_time;
+				start_column = 0;
 			}
+			else
+			{
+				start_column = (guint)round((epg_event.start_time - start_time) / COLUMNS_PER_HOUR);
+			}
+		
+			guint end_column = (guint)round((event_end_time - start_time) / COLUMNS_PER_HOUR);
+			if (end_column > number_columns-1)
+			{
+				end_column = number_columns-1;
+			}
+		
+			guint column_count = end_column - start_column;
+			if (start_column >= total_number_columns && column_count > 0)
+			{
+				// If there's a gap, plug it
+				if (start_column > total_number_columns)
+				{
+					guint empty_columns = start_column - total_number_columns;
+					Gtk::Button& button = attach_button(
+						empty_columns < 10 ? _("-") : _("Unknown program"), false,
+						total_number_columns + 1, start_column + 1, table_row, table_row + 1);
+					button.set_sensitive(false);
+					total_number_columns += empty_columns;
+				}
+			
+				if (column_count > 0)
+				{
+					guint converted_start_time = convert_to_utc_time(epg_event.start_time);
+
+					Glib::ustring text;
+					if (show_epg_time)
+					{
+						text = get_local_time_text(converted_start_time, "<b>%H:%M");
+						text += get_local_time_text(converted_start_time + epg_event.duration, " - %H:%M</b>\n");
+					}
+					text += encode_xml(epg_event.get_title());
+
+					gboolean record = scheduled_recording_manager.is_recording(epg_event);
+					
+					Gtk::Button& button = attach_button(text, record, start_column + 1, end_column + 1, table_row, table_row + 1);
+					button.signal_clicked().connect(
+						sigc::bind<EpgEvent>
+						(
+							sigc::mem_fun(*this, &GtkEpgWidget::on_button_program_clicked),
+							epg_event
+						)
+					);
+					button.signal_button_press_event().connect(
+						sigc::bind<EpgEvent>
+						(
+							sigc::mem_fun(*this, &GtkEpgWidget::on_button_program_press_event),
+							epg_event
+						),
+					    false
+					);
+
+					if (show_epg_tooltips)
+					{
+						Glib::ustring tooltip_text = get_local_time_text(converted_start_time, "%A, %B %d\n%H:%M");
+						tooltip_text += get_local_time_text(converted_start_time + epg_event.duration, " - %H:%M");
+						Glib::ustring subtitle = trim_string(epg_event.get_subtitle());
+						Glib::ustring description = trim_string(epg_event.get_description());
+						if (!subtitle.empty())
+						{
+							tooltip_text += "\n" + subtitle;
+							if (!description.empty())
+							{
+								tooltip_text += " - ";
+							}
+						}
+						else if (!description.empty())
+						{
+							tooltip_text += "\n";
+						}
+						tooltip_text += description;
+						button.set_tooltip_text(tooltip_text);
+					}
+				}
+
+				total_number_columns += column_count;
+			}
+			last_event_end_time = event_end_time;
 		}
 	}
 	
@@ -367,9 +355,11 @@ void GtkEpgWidget::create_channel_row(const Channel& const_channel,
 	}
 }
 
-Gtk::ToggleButton& GtkEpgWidget::attach_toggle_button(const Glib::ustring& text, gboolean record, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options)
+Gtk::RadioButton& GtkEpgWidget::attach_radio_button(Gtk::RadioButtonGroup& group, const Glib::ustring& text, gboolean record, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options)
 {
-	Gtk::ToggleButton* button = new Gtk::ToggleButton();
+	Gtk::RadioButton* button = Gtk::manage(new Gtk::RadioButton(group));
+	button->property_draw_indicator() = false;
+	button->set_active(false);
 	button->set_alignment(0, 0.5);
 
 	Gtk::HBox* hbox = Gtk::manage(new Gtk::HBox());
@@ -377,6 +367,7 @@ Gtk::ToggleButton& GtkEpgWidget::attach_toggle_button(const Glib::ustring& text,
 	Gtk::Label* label = Gtk::manage(new Gtk::Label(text));
 	label->set_use_markup(true);
 	label->set_alignment(0, 0.5);
+	label->set_padding(3, 0);
 	hbox->pack_start(*label, true, true);
 
 	if (record == true)
@@ -386,7 +377,7 @@ Gtk::ToggleButton& GtkEpgWidget::attach_toggle_button(const Glib::ustring& text,
 		hbox->pack_end(*image, false, false);
 	}
 	
-	button->add(*hbox);	
+	button->add(*hbox);
 	button->show_all();
 
 	attach_widget(*button, left_attach, right_attach, top_attach, bottom_attach, attach_options);
@@ -435,23 +426,15 @@ void GtkEpgWidget::attach_widget(Gtk::Widget& widget, guint left_attach, guint r
 	widget.show();
 }
 
-void GtkEpgWidget::on_button_channel_clicked(guint channel_id)
-{
-	TRY
-	get_application().set_display_channel_by_id(channel_id);
-	CATCH
-
-	TRY
-	update_table();
-	CATCH
-}
-
-bool GtkEpgWidget::on_button_channel_press_event(GdkEventButton* event, Channel& channel)
+bool GtkEpgWidget::on_button_channel_press_event(GdkEventButton* event, guint channel_id)
 {
+	device_manager.check_frontend();
+		
 	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
 	{
-		TRY
-		if (get_application().stream_manager.is_recording(channel))
+		Channel& channel = channel_manager.get_channel_by_id(channel_id);
+
+		if (stream_manager.is_recording(channel))
 		{
 			get_application().stop_recording(channel);
 		}
@@ -459,11 +442,12 @@ bool GtkEpgWidget::on_button_channel_press_event(GdkEventButton* event, Channel&
 		{
 			get_application().start_recording(channel);
 		}
-		CATCH
 
-		TRY
 		update_table();
-		CATCH
+	}
+	else if (event->button == 1)
+	{
+		signal_start_display(channel_id);
 	}
 
 	return false;
@@ -473,10 +457,16 @@ bool GtkEpgWidget::on_button_program_press_event(GdkEventButton* event, EpgEvent
 {
 	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
 	{
-		ScheduledRecordingDialog& scheduled_recording_dialog = ScheduledRecordingDialog::create(builder);
-		scheduled_recording_dialog.run(MainWindow::create(builder), epg_event);
-		scheduled_recording_dialog.hide();
-		get_application().update();
+		if (scheduled_recording_manager.is_recording(epg_event))
+		{
+			scheduled_recording_manager.remove_scheduled_recording(epg_event);
+		}
+		else
+		{
+			scheduled_recording_manager.set_scheduled_recording(epg_event);
+		}
+
+		signal_update();
 	}
 
 	return false;
@@ -484,36 +474,5 @@ bool GtkEpgWidget::on_button_program_press_event(GdkEventButton* event, EpgEvent
 
 void GtkEpgWidget::on_button_program_clicked(EpgEvent& epg_event)
 {
-	TRY
-		
-	FullscreenBugWorkaround fullscreen_bug_workaround;
-
-	Gtk::Dialog* dialog_program_details = NULL;
-	builder->get_widget("dialog_program_details", dialog_program_details);
-
-	Gtk::TextView* text_view = NULL;
-	builder->get_widget("text_view_program_title", text_view);
-	text_view->get_buffer()->assign(epg_event.get_title());
-
-	builder->get_widget("text_view_program_description", text_view);
-	text_view->get_buffer()->assign(epg_event.get_description());
-
-	builder->get_widget("text_view_program_start_time", text_view);
-	text_view->get_buffer()->assign(epg_event.get_start_time_text());
-
-	builder->get_widget("text_view_program_duration", text_view);
-	text_view->get_buffer()->assign(epg_event.get_duration_text());
-
-	gint result = dialog_program_details->run();
-	dialog_program_details->hide();
-
-	if (result == 1)
-	{
-		ScheduledRecordingDialog& scheduled_recording_dialog = ScheduledRecordingDialog::create(builder);
-		scheduled_recording_dialog.run(MainWindow::create(builder), epg_event);
-		scheduled_recording_dialog.hide();
-		get_application().update();
-	}
-	
-	CATCH
+	EpgEventDialog::create(builder).show_epg_event(epg_event);
 }
