@@ -21,7 +21,7 @@
 #ifndef __GTK_EPG_WIDGET__
 #define __GTK_EPG_WIDGET__
 
-#include <libgnomeuimm.h>
+#include <gtkmm.h>
 #include "data.h"
 #include "me-tv-ui.h"
 
@@ -34,8 +34,7 @@ private:
 	gsize								span_seconds;
 	const Glib::RefPtr<Gtk::Builder>	builder;
 	guint								epg_span_hours;
-	guint								epg_page;
-	IntComboBox*						combo_box_epg_page;
+	Gtk::SpinButton*					spin_button_epg_page;
 	Gtk::Label*							label_epg_page;
 	Gtk::Table*							table_epg;
 	Gtk::ScrolledWindow*				scrolled_window_epg;
@@ -44,19 +43,18 @@ private:
 	void next();
 		
 	bool on_button_program_press_event(GdkEventButton* event, EpgEvent& epg_event);
-	bool on_button_channel_press_event(GdkEventButton* event, Channel& channel);
+	bool on_button_channel_press_event(GdkEventButton* event, guint channel_id);
 	void on_button_program_clicked(EpgEvent& epg_event);
-	void on_button_channel_clicked(guint channel_id);
-	void on_combo_box_epg_page_changed();
+	void on_spin_button_epg_page_changed();
 		
 	void clear();
 	void update_pages();
 	
-	Gtk::ToggleButton& attach_toggle_button(const Glib::ustring& text, gboolean record, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options = Gtk::FILL);
+	Gtk::RadioButton& attach_radio_button(Gtk::RadioButtonGroup& group, const Glib::ustring& text, gboolean record, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options = Gtk::FILL);
 	Gtk::Button& attach_button(const Glib::ustring& text, gboolean record, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options = Gtk::FILL);
 	Gtk::Label& attach_label(const Glib::ustring& text, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options = Gtk::FILL);
 	void attach_widget(Gtk::Widget& widget, guint left_attach, guint right_attach, guint top_attach, guint bottom_attach, Gtk::AttachOptions attach_options = Gtk::FILL);
-	void create_channel_row(const Channel& const_channel,
+	void create_channel_row(Gtk::RadioButtonGroup& group, Channel& const_channel,
 		guint table_row, gboolean selected, guint start_time, guint channel_number,
 		gboolean show_channel_number, gboolean show_epg_time, gboolean show_epg_tooltips);
 
