@@ -23,52 +23,66 @@
 
 #include "me-tv.h"
 #include "gtk_epg_widget.h"
-#include "meters_dialog.h"
 #include "engine.h"
+#include <dbus/dbus.h>
+#include <gtkmm/volumebutton.h>
 
 typedef enum
 {
 	VIEW_MODE_VIDEO,
-	VIEW_MODE_CONTROLS,
-	VIEW_MODE_EPG
+	VIEW_MODE_CONTROLS
 } ViewMode;
 
 class MainWindow : public Gtk::Window
 {
 private:
-	const Glib::RefPtr<Gtk::Builder>		builder;
-	Gtk::DrawingArea*						drawing_area_video;
-	GtkEpgWidget*							widget_epg;
-	guint									last_motion_time;
-	GdkCursor*								hidden_cursor;
-	gboolean								is_cursor_visible;
-	Gtk::HScale*							h_scale_position;
-	Gtk::Statusbar*							statusbar;
-	ViewMode								view_mode;
-	ViewMode								prefullscreen_view_mode;
-	guint									last_update_time;
-	guint									last_poke_time;
-	guint									timeout_source;
-	Engine*									engine;
-	gint									output_fd;
-	Glib::StaticRecMutex					mutex;
-	gboolean								mute_state;
-	gboolean								maximise_forced;
-	guint									channel_change_timeout;
-	guint									temp_channel_number;
-
-	Glib::RefPtr<Gtk::UIManager>			ui_manager;
+	const Glib::RefPtr<Gtk::Builder>	builder;
+	Gtk::DrawingArea*					drawing_area_video;
+	GtkEpgWidget*						widget_epg;
+	guint								last_motion_time;
+	GdkCursor*							hidden_cursor;
+	gboolean							is_cursor_visible;
+	Gtk::MenuBar*						menu_bar;
+	Gtk::VolumeButton*					volume_button;
+	Gtk::HBox*							hbox_controls;
+	ViewMode							view_mode;
+	ViewMode							prefullscreen_view_mode;
+	guint								last_update_time;
+	guint								timeout_source;
+	Engine*								engine;
+	gint								output_fd;
+	Glib::StaticRecMutex				mutex;
+	gboolean							mute_state;
+	guint								channel_change_timeout;
+	guint								temp_channel_number;
+	guint								screensaver_inhibit_cookie;
 	
 	void stop();
 	void set_view_mode(ViewMode display_mode);
 	void load_devices();
-	void show_scheduled_recordings_dialog();
 	void set_state(const Glib::ustring& name, gboolean state);
 	void add_channel_number(guint channel_number);
-	void toggle_fullscreen();
 	void toggle_mute();
 	void set_mute_state(gboolean state);
-		
+	void set_status_text(const Glib::ustring& text);
+	void select_channel_to_play();
+
+	void play(const Glib::ustring& mrl);
+	void pause(gboolean state);
+	void restart_engine();
+	void start_engine();
+	void stop_engine();
+
+	void toggle_visibility();
+	void save_geometry();
+
+	void fullscreen(gboolean change_mode = true);
+	void unfullscreen(gboolean restore_mode = true);
+	gboolean is_fullscreen();
+
+	gboolean is_screensaver_inhibited() { return (screensaver_inhibit_cookie != 0); }
+	void inhibit_screensaver(gboolean activate);
+
 	bool on_delete_event(GdkEventAny* event);
 	bool on_motion_notify_event(GdkEventMotion* event);
 	bool on_drawing_area_expose_event(GdkEventExpose* event);
@@ -79,11 +93,35 @@ private:
 	void on_menu_item_audio_stream_activate(guint audio_stream_index);
 	void on_menu_item_subtitle_stream_activate(guint audio_stream_index);
 
+	void on_start_display(guint channel_id);
+	void on_stop_display();
+	void on_update();
+	void on_error(const Glib::ustring& message);
+
 	void on_show();
 	void on_hide();
-	void start_engine();
-	void stop_engine();
 			
+	void on_about();
+	void on_auto_record();
+	void on_audio_channel_both();
+	void on_audio_channel_left();
+	void on_audio_channel_right();
+	void on_button_volume_value_changed(double value);
+	void on_change_view_mode();
+	void on_channels();
+	void on_decrease_volume();
+	void on_devices();
+	void on_epg_event_search();
+	void on_fullscreen();
+	void on_increase_volume();
+	void on_meters();
+	void on_mute();
+	void on_next_channel();
+	void on_preferences();
+	void on_previous_channel();
+	void on_present();
+	void on_scheduled_recordings();
+
 	void create_engine();
 public:
 	MainWindow(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
@@ -91,36 +129,13 @@ public:
 		
 	static MainWindow* create(Glib::RefPtr<Gtk::Builder> builder);
 		
-	void on_next_channel();
-	void on_previous_channel();
-	void on_change_view_mode();
-	void on_devices();
-	void on_channels();
-	void on_scheduled_recordings();
-	void on_meters();
-	void on_preferences();
-	void on_fullscreen();
-	void on_mute();
-	void on_audio_channel_both();
-	void on_audio_channel_left();
-	void on_audio_channel_right();
-	void on_about();
+	void on_exception();
 
-	void show_devices_dialog();
 	void show_channels_dialog();
 	void show_preferences_dialog();
-	
-	void toggle_visibility();
-	void update();
-	void save_geometry();
-
-	void play(const Glib::ustring& mrl);
-	void pause(gboolean state);
-	void restart_engine();
-
-	void fullscreen(gboolean change_mode = true);
-	void unfullscreen(gboolean restore_mode = true);
-	gboolean is_fullscreen();
+	void show_scheduled_recordings_dialog();
+	void show_epg_event_search_dialog();
+	void show_error(const Glib::ustring& message);
 };
 
 #endif
