@@ -24,12 +24,22 @@
 #include <glibmm.h>
 #include <list>
 
+#define SCHEDULED_RECORDING_ACTION_AFTER_NONE		0
+#define SCHEDULED_RECORDING_ACTION_AFTER_CLOSE		1
+#define SCHEDULED_RECORDING_ACTION_AFTER_SHUTDOWN	2
+
+#define SCHEDULED_RECORDING_RECURRING_TYPE_ONCE			0
+#define SCHEDULED_RECORDING_RECURRING_TYPE_EVERYDAY		1
+#define SCHEDULED_RECORDING_RECURRING_TYPE_EVERYWEEK	2
+#define SCHEDULED_RECORDING_RECURRING_TYPE_EVERYWEEKDAY	3
+
 class ScheduledRecording
 {
 public:
 	guint			scheduled_recording_id;
 	Glib::ustring	description;
-	guint			type;
+	guint			recurring_type;
+	guint			action_after;
 	guint			channel_id;
 	guint			start_time;
 	guint			duration;
