@@ -31,19 +31,27 @@ class ScheduledRecordingManager
 {
 private:
 	Glib::StaticRecMutex	mutex;
+
+	gboolean dirty;
+
+	gboolean is_device_available(const Glib::ustring& device, const ScheduledRecording& scheduled_recording);
+	void select_device(ScheduledRecording& scheduled_recording);
+
 public:
-	ScheduledRecordingManager();
+	void initialise();
 		
 	ScheduledRecordingList scheduled_recordings;
 
 	void load(Data::Connection& connection);
 	void save(Data::Connection& connection);
 
+	void set_scheduled_recording(EpgEvent& epg_event);
 	void set_scheduled_recording(ScheduledRecording& scheduled_recording);
 	void remove_scheduled_recording(guint scheduled_recording_id);		
+	void remove_scheduled_recording(EpgEvent& epg_event);
 	ScheduledRecording get_scheduled_recording(guint scheduled_recording_id);
 	ScheduledRecordingList check_scheduled_recordings();
-	gboolean is_recording(const Channel& channel);
+	guint is_recording(const Channel& channel);
 	gboolean is_recording(const EpgEvent& epg_event);
 };
 
